@@ -1,4 +1,4 @@
-import os, sys, io, unittest, json, enum, collections as cs
+import os, sys, io, re, unittest, json, enum, textwrap, collections as cs
 
 import yaml
 
@@ -7,6 +7,9 @@ except ImportError:
 	sys.path.insert(1, os.path.join(__file__, *['..']*3))
 	import pyaml
 
+try: import unidecode
+except ImportError: unidecode = None
+
 
 large_yaml = br'''
 ### Default (baseline) configuration parameters.
@@ -169,7 +172,7 @@ data = dict(
 data['query_dump_clone'] = data['query_dump']
 data['ids']['id в уникоде'] = [4, 5, 6]
 data['ids']['id2 в уникоде'] = data['ids']['id в уникоде']
-# data["'asd'\n!\0\1"] =cs.OrderedDict([('b', 1), ('a', 2)]) <-- fails in many ways
+data["'asd'\n!\0\1"] = cs.OrderedDict([('b', 1), ('a', 2)])
 
 data_str_multiline = dict(cert=(
 	'-----BEGIN CERTIFICATE-----\n'
@@ -216,6 +219,10 @@ data_str_long = dict(cert=(
 
 class DumpTests(unittest.TestCase):
 
+	def yaml_var(self, ys, raw=False):
+		ys = textwrap.dedent(ys).replace('\t', '  ')
+		return ys if raw else yaml.safe_load(ys)
+
 	def flatten(self, data, path=tuple()):
 		dst = list()
 		if isinstance(data, (tuple, list)):
@@ -225,6 +232,18 @@ class DumpTests(unittest.TestCase):
 		else: dst.append((path, data))
 		return tuple(sorted(dst, key=lambda v: json.dumps(v, sort_keys=True)))
 
+	def pos_list(self, ys, sep='\n'):
+		pos, pos_list = 0, list()
+		while True:
+			pos = ys.find(sep, pos+1)
+			if pos < 0: break
+			pos_list.append(pos)
+		return pos_list
+
+	def empty_line_list(self, ys):
+		return list(n for n, line in enumerate(ys.splitlines()) if not line)
+
+
 	def test_dst(self):
 		buff = io.BytesIO()
 		self.assertIs(pyaml.dump(data, buff), None)
@@ -241,12 +260,7 @@ class DumpTests(unittest.TestCase):
 		a = self.flatten(data)
 		b = pyaml.dump(data, vspacing=dict(split_lines=10, split_count=2))
 		self.assertEqual(a, self.flatten(yaml.safe_load(b)))
-		pos, pos_list = 0, list()
-		while True:
-			pos = b.find(u'\n', pos+1)
-			if pos < 0: break
-			pos_list.append(pos)
-		self.assertEqual( pos_list,
+		self.assertEqual( self.pos_list(b, '\n'),
 			[12, 13, 25, 33, 52, 73, 88, 107, 157, 184, 264, 299, 344, 345, 355, 375, 399,
 			424, 425, 458, 459, 467, 505, 561, 600, 601, 607, 660, 681, 705, 738, 767, 795,
 			796, 805, 806, 820, 831, 866, 936, 937, 949, 950, 963, 998, 1021, 1041, 1072,
@@ -261,7 +275,16 @@ class DumpTests(unittest.TestCase):
 		b = pyaml.dump(data, force_embed=False)
 		self.assertNotIn('&id00', b)
 		self.assertIn('query_dump_clone: *query_dump_clone', b)
-		self.assertIn("id в уникоде: &ids_-_id2_v_unikode", b) # kinda bug - should be just "id"
+		self.assertIn('id в уникоде: &ids_-_id2_', b)
+		if not unidecode: self.assertIn('id в уникоде: &ids_-_id2__id00', b)
+
+	def test_ids_unidecode(self):
+		if not unidecode:
+			self.skipTest('No unidecode module to test ids from non-ascii keys')
+		b = pyaml.dump(data, force_embed=False)
+		self.assertNotIn('&id00', b)
+		self.assertNotIn('_id00', b)
+		self.assertIn('id в уникоде: &ids_-_id2_v_unikode', b)
 
 	def test_force_embed(self):
 		for check, fe in (self.assertNotIn, True), (self.assertIn, False):
@@ -380,7 +403,7 @@ class DumpTests(unittest.TestCase):
 		TestTuple = cs.namedtuple('TestTuple', 'y x z')
 		val = TestTuple(1, 2, 3)
 		val_str = pyaml.dump(val, sort_keys=False)
-		self.assertEqual(val_str, u'y: 1\nx: 2\nz: 3\n') # namedtuple order was preserved
+		self.assertEqual(val_str, 'y: 1\nx: 2\nz: 3\n') # namedtuple order was preserved
 
 	def test_ordereddict(self):
 		d = cs.OrderedDict((i, '') for i in reversed(range(10)))
@@ -412,7 +435,6 @@ class DumpTests(unittest.TestCase):
 		self.assertTrue(docs_str.startswith('---'))
 		self.assertIn('---\n\na: 1\n\nb: 2\n\nc: 3\n', docs_str)
 
-
 		docs_str2 = pyaml.dump(docs, vspacing=True, multiple_docs=True)
 		self.assertEqual(docs_str, docs_str2)
 		docs_str2 = pyaml.dump(docs, vspacing=True)
@@ -425,10 +447,103 @@ class DumpTests(unittest.TestCase):
 
 	def test_ruamel_yaml(self):
 		try: from ruamel.yaml import YAML
-		except ImportError: return unittest.skip('No ruamel.yaml module to test it')
+		except ImportError: self.skipTest('No ruamel.yaml module to test it')
 		data = YAML(typ='safe').load(large_yaml)
 		yaml_str = pyaml.dump(data)
 
+	def test_dump_stream_kws(self):
+		data = [1, 2, 3]
+		buff1, buff2 = io.StringIO(), io.StringIO()
+		pyaml.dump(data, dst=buff1)
+		pyaml.dump(data, stream=buff2)
+		self.assertEqual(buff1.getvalue(), buff2.getvalue())
+
+		buff1.seek(0); buff1.truncate()
+		pyaml.dump(data, dst=buff1, stream=buff1)
+		self.assertEqual(buff1.getvalue(), buff2.getvalue())
+		ys = pyaml.dump(data, dst=str, stream=str)
+		self.assertEqual(ys, buff2.getvalue())
+
+		buff1.seek(0); buff1.truncate(); buff2.seek(0); buff2.truncate()
+		with self.assertRaises(TypeError):
+			pyaml.dump(data, dst=buff1, stream=buff2)
+		with self.assertRaises(TypeError):
+			pyaml.dump(data, dst=str, stream=buff2)
+		self.assertEqual(buff1.getvalue(), '')
+		self.assertEqual(buff2.getvalue(), '')
+
+	def test_list_vspacing(self):
+		itm = self.yaml_var('''
+			builtIn: 1
+			datasource:
+				type: grafana
+				uid: -- Grafana --
+			enable: yes
+			hide: yes
+			iconColor: rgba(0, 211, 255, 1)
+			name: Annotations & Alerts
+			type: dashboard''')
+		ys = pyaml.dump(dict(mylist=[itm]*10))
+		self.assertEqual(
+			self.empty_line_list(ys),
+			[1, 11, 21, 31, 41, 51, 61, 71, 81, 91] )
+
+		ys = self.yaml_var('''
+			panels:
+				- datasource:
+						type: datasource
+						uid: grafana
+					fieldConfig:''', raw=True)
+		for n in range(60): ys += '\n' + '  '*3 + f'field{n}: value-{n}'
+		ys = pyaml.dump(yaml.safe_load(ys))
+		self.assertEqual(self.empty_line_list(ys), list(range(4, 126, 2)))
+
+	def test_anchor_cutoff(self):
+		data = self.yaml_var('''
+			similique-natus-inventore-deserunt-amet-explicabo-cum-accusamus-temporibus:
+				quam-nulla-dolorem-dolore-velit-quis-deserunt-est-ullam-exercitationem:
+					culpa-quia-incidunt-accusantium-ad-dicta-nobis-rerum-veritatis: &test
+						test: 1
+			similique-commodi-aperiam-libero-error-eos-quidem-eius:
+				ipsam-labore-enim,-vero-voluptatem-eaque-dolores-blanditiis-recusandae:
+					quas-atque-maxime-itaque-ullam-sequi-suscipit-quis-vitae-veritatis: *test''')
+		ys = pyaml.dump(data, force_embed=False)
+		for c in '&', r'\*':
+			self.assertTrue(m := re.search(fr'(?<= ){c}\S+', ys))
+			self.assertLess(len(m[0]), 50)
+			self.assertIn('similique', m[0])
+			self.assertIn('veritatis', m[0])
+
+		data = dict(test1=dict(test2=(v := dict(a=1, b=2, c=3))), test3=dict(test4=v))
+		ys = pyaml.dump(data, force_embed=False)
+		for c in '&', r'\*':
+			self.assertTrue(m := re.search(fr'(?<= ){c}\S+', ys))
+			self.assertLess(len(m[0]), 30)
+			self.assertEqual(len(re.findall(r'test\d', m[0])), 2)
+
+	def test_group_online_values(self):
+		data = self.yaml_var('''
+			similique-natus: 1
+			similique-commodi:
+				aperiam-libero: 2
+			"111": digit-string
+			deserunt-est-2: asdasd
+			deserunt-est-1: |
+				line1
+				line2
+			culpa-quia: 1234
+			deserunt-est-3: asdasd
+			10: test1
+			200: test
+			30: test2''')
+		ys1 = pyaml.dump(data,
+			sort_dicts=pyaml.PYAMLSort.oneline_group,
+			vspacing=dict(split_lines=0, split_count=0) )
+		self.assertEqual(self.empty_line_list(ys1), [8, 12])
+		ys2 = pyaml.dump(data, vspacing=dict(split_lines=0, split_count=0))
+		self.assertNotEqual(ys1, ys2)
+		self.assertEqual(self.empty_line_list(ys2), [1, 3, 5, 7, 9, 13, 15, 17, 19, 21])
+
 
 if __name__ == '__main__':
 	unittest.main()
